import 'package:mockito/mockito.dart';
import 'package:flutter/services.dart';
import 'package:flutter_test/flutter_test.dart';
import 'package:plugin_platform_interface/plugin_platform_interface.dart';

import 'package:libbase_interface/base_method_channel.dart';
import 'package:libbase_interface/base_interface.dart';

void main() {
  TestWidgetsFlutterBinding.ensureInitialized();

  group('BaseInterface', () {
    test('BaseMethodChannel() is the default instance', () {
      expect(BaseInterface.instance, isInstanceOf<BaseMethodChannel>());
    });

    test('Cannot be implemented with `implements`', () {
      expect(() {
        BaseInterface.instance = ImplementsBaseInterface();
      }, throwsA(isInstanceOf<AssertionError>()));
    });

    test('Can be mocked with `implements`', () {
      final BaseInterfaceMock mock = BaseInterfaceMock();
      BaseInterface.instance = mock;
    });

    test('Can be extended', () {
      BaseInterface.instance = ExtendsBaseInterface();
    });
  });

  group('$BaseMethodChannel', () {
    const MethodChannel channel =
        MethodChannel('plugins.flutter.io/url_launcher');
    final List<MethodCall> log = <MethodCall>[];
    channel.setMockMethodCallHandler((MethodCall methodCall) async {
      log.add(methodCall);
    });

    final BaseMethodChannel launcher = BaseMethodChannel();

    tearDown(() {
      log.clear();
    });

    test('should navigate', () async {
      await launcher.navigate('http://mock.com/');
      expect(
        log,
        <Matcher>[
          isMethodCall('navigate', arguments: <String, Object>{
            'url': 'http://mock.com/',
          })
        ],
      );
    });
  });
}

class BaseInterfaceMock extends Mock
    with MockPlatformInterfaceMixin
    implements BaseInterface {}

class ImplementsBaseInterface extends Mock implements BaseInterface {}

class ExtendsBaseInterface extends BaseInterface {}
